package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            findEmployeeOrderBySalary();
            findEmployeeOrderByDeptAndSalary();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 2000, "IT");
        Employee employee2 = Employee.create("Rose", 3500, "Admin");
        Employee employee3 = Employee.create("Denise", 2500, "Admin");
        Employee employee4 = Employee.create("Mike", 4000, "IT");
        Employee employee5 = Employee.create("Linda", 4500, "Sales");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.persist(employee5);
        em.getTransaction().commit();
        em.close();
    }

    private static void findEmployeeOrderBySalary() {
        System.out.println("-- All employees order by salary --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cb.createQuery(Employee.class);//query object
        Root<Employee> employee = query.from(Employee.class);//from part
        query.select(employee);//'select from' part
        query.orderBy(cb.desc(employee.get(Employee_.salary)));//order by part
        TypedQuery<Employee> typedQuery = em.createQuery(query);
        typedQuery.getResultList()
                  .forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeOrderByDeptAndSalary() {
        System.out.println("-- All employees order by dept and salary --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cb.createQuery(Employee.class);//query object
        Root<Employee> employee = query.from(Employee.class);//from part
        query.select(employee);//'select from' part
        query.orderBy(
                cb.asc(employee.get(Employee_.dept)),
                cb.desc(employee.get(Employee_.salary))
        );//order by part
        TypedQuery<Employee> typedQuery = em.createQuery(query);
        typedQuery.getResultList()
                  .forEach(System.out::println);
        em.close();
    }
}